# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 2.14.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

import curia.api.curia.api.curia.api.curia.api.swagger_client
from curia.api.curia.api.curia.api.curia.api.swagger_client.api.datasets_api import DatasetsApi  # noqa: E501
from curia.api.curia.api.curia.api.curia.api.swagger_client.rest import ApiException


class TestDatasetsApi(unittest.TestCase):
    """DatasetsApi unit test stubs"""

    def setUp(self):
        self.api = DatasetsApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_create_many_base_dataset_controller_dataset(self):
        """Test case for create_many_base_dataset_controller_dataset

        Create many Dataset  # noqa: E501
        """
        pass

    def test_create_one_base_dataset_controller_dataset(self):
        """Test case for create_one_base_dataset_controller_dataset

        Create one Dataset  # noqa: E501
        """
        pass

    def test_dataset_controller_calculate_stats(self):
        """Test case for dataset_controller_calculate_stats

        Calculate dataset stats  # noqa: E501
        """
        pass

    def test_dataset_controller_download_dataset(self):
        """Test case for dataset_controller_download_dataset

        Download dataset  # noqa: E501
        """
        pass

    def test_dataset_controller_get_datasets_by_tag(self):
        """Test case for dataset_controller_get_datasets_by_tag

        Retrieve datasets by tag  # noqa: E501
        """
        pass

    def test_dataset_controller_internal_download_dataset(self):
        """Test case for dataset_controller_internal_download_dataset

        Download dataset  # noqa: E501
        """
        pass

    def test_dataset_controller_upload(self):
        """Test case for dataset_controller_upload

        Upload dataset  # noqa: E501
        """
        pass

    def test_dataset_controller_upload_abort(self):
        """Test case for dataset_controller_upload_abort

        cancel multipart upload for large dataset  # noqa: E501
        """
        pass

    def test_dataset_controller_upload_finish(self):
        """Test case for dataset_controller_upload_finish

        complete upload for large dataset using multipart upload  # noqa: E501
        """
        pass

    def test_dataset_controller_upload_multipart_start(self):
        """Test case for dataset_controller_upload_multipart_start

        Initiate upload for large dataset using multipart upload  # noqa: E501
        """
        pass

    def test_delete_one_base_dataset_controller_dataset(self):
        """Test case for delete_one_base_dataset_controller_dataset

        Delete one Dataset  # noqa: E501
        """
        pass

    def test_get_many_base_dataset_controller_dataset(self):
        """Test case for get_many_base_dataset_controller_dataset

        Retrieve many Dataset  # noqa: E501
        """
        pass

    def test_get_one_base_dataset_controller_dataset(self):
        """Test case for get_one_base_dataset_controller_dataset

        Retrieve one Dataset  # noqa: E501
        """
        pass

    def test_replace_one_base_dataset_controller_dataset(self):
        """Test case for replace_one_base_dataset_controller_dataset

        Replace one Dataset  # noqa: E501
        """
        pass

    def test_update_one_base_dataset_controller_dataset(self):
        """Test case for update_one_base_dataset_controller_dataset

        Update one Dataset  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from curia.api.swagger_client.api.cohorts_api import CohortsApi
from curia.api.swagger_client.api.data_queries_api import DataQueriesApi
from curia.api.swagger_client.api.data_stores_api import DataStoresApi
from curia.api.swagger_client.api.data_tables_api import DataTablesApi
from curia.api.swagger_client.api.databases_api import DatabasesApi
from curia.api.swagger_client.api.dataset_columns_api import DatasetColumnsApi
from curia.api.swagger_client.api.datasets_api import DatasetsApi
from curia.api.swagger_client.api.feature_stores_api import FeatureStoresApi
from curia.api.swagger_client.api.features_api import FeaturesApi
from curia.api.swagger_client.api.internal_api import InternalApi
from curia.api.swagger_client.api.model_job_outputs_api import ModelJobOutputsApi
from curia.api.swagger_client.api.model_job_statuses_api import ModelJobStatusesApi
from curia.api.swagger_client.api.model_jobs_api import ModelJobsApi
from curia.api.swagger_client.api.model_populations_api import ModelPopulationsApi
from curia.api.swagger_client.api.models_api import ModelsApi
from curia.api.swagger_client.api.organization_settings_api import OrganizationSettingsApi
from curia.api.swagger_client.api.organizations_api import OrganizationsApi
from curia.api.swagger_client.api.platform_api import PlatformApi
from curia.api.swagger_client.api.population_stores_api import PopulationStoresApi
from curia.api.swagger_client.api.project_members_api import ProjectMembersApi
from curia.api.swagger_client.api.projects_api import ProjectsApi
from curia.api.swagger_client.api.scheduler_api import SchedulerApi
from curia.api.swagger_client.api.tasks_api import TasksApi
from curia.api.swagger_client.api.user_audit_trail_api import UserAuditTrailApi
from curia.api.swagger_client.api.user_favorites_api import UserFavoritesApi
from curia.api.swagger_client.api.user_management_api import UserManagementApi
from curia.api.swagger_client.api.workflows_api import WorkflowsApi



import abc
import functools
import time

from curia.session import Session
from curia.utils.string import to_camel_case


class JobInterface(abc.ABC):
    _METHOD_PATTERNS = {
        "get": "get_one_base_{0}_controller_{0}",
        "start": "{0}_controller_start",
        "stop": "{0}_controller_stop"
    }
    SUPPORTED_FUNCTIONS = list(_METHOD_PATTERNS.keys())

    SUPPORTED_AUTOVARIANTS = ['model_job']

    def __init__(self, session: Session):
        self.session = session

    def run(self, job_id, timeout=3600, interval=5):
        self.start(job_id)
        start_time = time.time()
        while True:
            if time.time() >= timeout + start_time:
                raise TimeoutError(f"Run operation timed out after {timeout} seconds")
            time.sleep(interval)
            job = self.get(job_id)
            status = job.status
            if status != "RUNNING":
                return job

    @abc.abstractmethod
    def get(self, job_id):
        pass

    @abc.abstractmethod
    def start(self, job_id):
        pass

    @abc.abstractmethod
    def stop(self, job_id):
        pass

    @classmethod
    def functionality_builder(cls, job_type_stub, function):
        def handler(self, job_id):
            return getattr(self.session.api_instance, cls._METHOD_PATTERNS[function].format(job_type_stub))(id=job_id)
        return handler

    @classmethod
    @functools.lru_cache(maxsize=None)
    def auto_variant(cls, job_type_stub):
        assert job_type_stub in cls.SUPPORTED_AUTOVARIANTS
        return type(
            f"{to_camel_case(job_type_stub).capitalize()}Interface",
            (cls,),
            {
                function: cls.functionality_builder(job_type_stub, function)
                for function in cls.SUPPORTED_FUNCTIONS
            }
        )


def get_interface(session: Session, job_type_stub: str = None, **kwargs) -> JobInterface:
    """
    :param session: Curia Session object to construct the interface around
    :param job_type_stub: Optional parameter specifying the job type stub to use.
    :param kwargs: Keyword arguments mapping job type stubs to boolean values. The first job_type_stub mapped to a
    truthy value will be used.
    :return: Instantiated JobInterface

    Example (the following are equivalent)::

        job_interface = get_interface(session, job_type_stub="model_job")

        job_interface = get_interface(session, model_job=True)

        job_interface = get_interface(session, model_job="2198302")

        job_interface = get_interface(session, model_job="2198302", process_job=False)

        job_interface = get_interface(session, model_job="2198302", process_job=0)

    """
    truthy_keys = [key for key, value in kwargs.items() if value]
    if job_type_stub is None:
        if len(truthy_keys) == 0:
            raise ValueError("No job_type_stub provided and no non-none job_type_stub kwargs provided!")
        if len(truthy_keys) > 1:
            truthy_keys_to_values = {key: kwargs[key] for key in truthy_keys}
            raise ValueError(f"No job_type_stub provided but multiple truthy job_type_stub kwargs provided! "
                             f"({truthy_keys_to_values})")
        job_type_stub: str = truthy_keys[0]
    else:
        if len(truthy_keys) > 0:
            truthy_keys_to_values = {key: kwargs[key] for key in truthy_keys}
            raise ValueError(f"job_type_stub provided and truthy job_type_stub kwargs also provided!"
                             f"(job_type_stub={job_type_stub}, truthy kwargs={truthy_keys_to_values})")
    variant: type = JobInterface.auto_variant(job_type_stub)
    return variant(session)


ModelJobInterface = JobInterface.auto_variant("model_job")



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.9.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class OrganizationSetting(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'models': 'object',
        'min_claim_date': 'datetime',
        'max_claim_date': 'datetime',
        'max_cohort_windows': 'float',
        'pipeline_version': 'str',
        'risk_version': 'str',
        'impactability_version': 'str',
        'primary_database': 'str',
        'features_database': 'str',
        'organization_id': 'str',
        'organization': 'Organization',
        'last_updated_by': 'str',
        'created_by': 'str',
        'created_at': 'datetime',
        'updated_at': 'datetime',
        'archived_at': 'datetime',
        'version': 'float'
    }

    attribute_map = {
        'id': 'id',
        'models': 'models',
        'min_claim_date': 'minClaimDate',
        'max_claim_date': 'maxClaimDate',
        'max_cohort_windows': 'maxCohortWindows',
        'pipeline_version': 'pipelineVersion',
        'risk_version': 'riskVersion',
        'impactability_version': 'impactabilityVersion',
        'primary_database': 'primaryDatabase',
        'features_database': 'featuresDatabase',
        'organization_id': 'organizationId',
        'organization': 'organization',
        'last_updated_by': 'lastUpdatedBy',
        'created_by': 'createdBy',
        'created_at': 'createdAt',
        'updated_at': 'updatedAt',
        'archived_at': 'archivedAt',
        'version': 'version'
    }

    def __init__(self, id=None, models=None, min_claim_date=None, max_claim_date=None, max_cohort_windows=None, pipeline_version=None, risk_version=None, impactability_version=None, primary_database=None, features_database=None, organization_id=None, organization=None, last_updated_by=None, created_by=None, created_at=None, updated_at=None, archived_at=None, version=None):  # noqa: E501
        """OrganizationSetting - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._models = None
        self._min_claim_date = None
        self._max_claim_date = None
        self._max_cohort_windows = None
        self._pipeline_version = None
        self._risk_version = None
        self._impactability_version = None
        self._primary_database = None
        self._features_database = None
        self._organization_id = None
        self._organization = None
        self._last_updated_by = None
        self._created_by = None
        self._created_at = None
        self._updated_at = None
        self._archived_at = None
        self._version = None
        self.discriminator = None
        if id is not None:
            self.id = id
        if models is not None:
            self.models = models
        if min_claim_date is not None:
            self.min_claim_date = min_claim_date
        if max_claim_date is not None:
            self.max_claim_date = max_claim_date
        self.max_cohort_windows = max_cohort_windows
        if pipeline_version is not None:
            self.pipeline_version = pipeline_version
        if risk_version is not None:
            self.risk_version = risk_version
        if impactability_version is not None:
            self.impactability_version = impactability_version
        if primary_database is not None:
            self.primary_database = primary_database
        if features_database is not None:
            self.features_database = features_database
        self.organization_id = organization_id
        if organization is not None:
            self.organization = organization
        if last_updated_by is not None:
            self.last_updated_by = last_updated_by
        if created_by is not None:
            self.created_by = created_by
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at
        if archived_at is not None:
            self.archived_at = archived_at
        if version is not None:
            self.version = version

    @property
    def id(self):
        """Gets the id of this OrganizationSetting.  # noqa: E501


        :return: The id of this OrganizationSetting.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrganizationSetting.


        :param id: The id of this OrganizationSetting.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def models(self):
        """Gets the models of this OrganizationSetting.  # noqa: E501


        :return: The models of this OrganizationSetting.  # noqa: E501
        :rtype: object
        """
        return self._models

    @models.setter
    def models(self, models):
        """Sets the models of this OrganizationSetting.


        :param models: The models of this OrganizationSetting.  # noqa: E501
        :type: object
        """

        self._models = models

    @property
    def min_claim_date(self):
        """Gets the min_claim_date of this OrganizationSetting.  # noqa: E501


        :return: The min_claim_date of this OrganizationSetting.  # noqa: E501
        :rtype: datetime
        """
        return self._min_claim_date

    @min_claim_date.setter
    def min_claim_date(self, min_claim_date):
        """Sets the min_claim_date of this OrganizationSetting.


        :param min_claim_date: The min_claim_date of this OrganizationSetting.  # noqa: E501
        :type: datetime
        """

        self._min_claim_date = min_claim_date

    @property
    def max_claim_date(self):
        """Gets the max_claim_date of this OrganizationSetting.  # noqa: E501


        :return: The max_claim_date of this OrganizationSetting.  # noqa: E501
        :rtype: datetime
        """
        return self._max_claim_date

    @max_claim_date.setter
    def max_claim_date(self, max_claim_date):
        """Sets the max_claim_date of this OrganizationSetting.


        :param max_claim_date: The max_claim_date of this OrganizationSetting.  # noqa: E501
        :type: datetime
        """

        self._max_claim_date = max_claim_date

    @property
    def max_cohort_windows(self):
        """Gets the max_cohort_windows of this OrganizationSetting.  # noqa: E501


        :return: The max_cohort_windows of this OrganizationSetting.  # noqa: E501
        :rtype: float
        """
        return self._max_cohort_windows

    @max_cohort_windows.setter
    def max_cohort_windows(self, max_cohort_windows):
        """Sets the max_cohort_windows of this OrganizationSetting.


        :param max_cohort_windows: The max_cohort_windows of this OrganizationSetting.  # noqa: E501
        :type: float
        """
        if max_cohort_windows is None:
            raise ValueError("Invalid value for `max_cohort_windows`, must not be `None`")  # noqa: E501

        self._max_cohort_windows = max_cohort_windows

    @property
    def pipeline_version(self):
        """Gets the pipeline_version of this OrganizationSetting.  # noqa: E501


        :return: The pipeline_version of this OrganizationSetting.  # noqa: E501
        :rtype: str
        """
        return self._pipeline_version

    @pipeline_version.setter
    def pipeline_version(self, pipeline_version):
        """Sets the pipeline_version of this OrganizationSetting.


        :param pipeline_version: The pipeline_version of this OrganizationSetting.  # noqa: E501
        :type: str
        """

        self._pipeline_version = pipeline_version

    @property
    def risk_version(self):
        """Gets the risk_version of this OrganizationSetting.  # noqa: E501


        :return: The risk_version of this OrganizationSetting.  # noqa: E501
        :rtype: str
        """
        return self._risk_version

    @risk_version.setter
    def risk_version(self, risk_version):
        """Sets the risk_version of this OrganizationSetting.


        :param risk_version: The risk_version of this OrganizationSetting.  # noqa: E501
        :type: str
        """

        self._risk_version = risk_version

    @property
    def impactability_version(self):
        """Gets the impactability_version of this OrganizationSetting.  # noqa: E501


        :return: The impactability_version of this OrganizationSetting.  # noqa: E501
        :rtype: str
        """
        return self._impactability_version

    @impactability_version.setter
    def impactability_version(self, impactability_version):
        """Sets the impactability_version of this OrganizationSetting.


        :param impactability_version: The impactability_version of this OrganizationSetting.  # noqa: E501
        :type: str
        """

        self._impactability_version = impactability_version

    @property
    def primary_database(self):
        """Gets the primary_database of this OrganizationSetting.  # noqa: E501


        :return: The primary_database of this OrganizationSetting.  # noqa: E501
        :rtype: str
        """
        return self._primary_database

    @primary_database.setter
    def primary_database(self, primary_database):
        """Sets the primary_database of this OrganizationSetting.


        :param primary_database: The primary_database of this OrganizationSetting.  # noqa: E501
        :type: str
        """

        self._primary_database = primary_database

    @property
    def features_database(self):
        """Gets the features_database of this OrganizationSetting.  # noqa: E501


        :return: The features_database of this OrganizationSetting.  # noqa: E501
        :rtype: str
        """
        return self._features_database

    @features_database.setter
    def features_database(self, features_database):
        """Sets the features_database of this OrganizationSetting.


        :param features_database: The features_database of this OrganizationSetting.  # noqa: E501
        :type: str
        """

        self._features_database = features_database

    @property
    def organization_id(self):
        """Gets the organization_id of this OrganizationSetting.  # noqa: E501


        :return: The organization_id of this OrganizationSetting.  # noqa: E501
        :rtype: str
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this OrganizationSetting.


        :param organization_id: The organization_id of this OrganizationSetting.  # noqa: E501
        :type: str
        """
        if organization_id is None:
            raise ValueError("Invalid value for `organization_id`, must not be `None`")  # noqa: E501

        self._organization_id = organization_id

    @property
    def organization(self):
        """Gets the organization of this OrganizationSetting.  # noqa: E501


        :return: The organization of this OrganizationSetting.  # noqa: E501
        :rtype: Organization
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """Sets the organization of this OrganizationSetting.


        :param organization: The organization of this OrganizationSetting.  # noqa: E501
        :type: Organization
        """

        self._organization = organization

    @property
    def last_updated_by(self):
        """Gets the last_updated_by of this OrganizationSetting.  # noqa: E501


        :return: The last_updated_by of this OrganizationSetting.  # noqa: E501
        :rtype: str
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, last_updated_by):
        """Sets the last_updated_by of this OrganizationSetting.


        :param last_updated_by: The last_updated_by of this OrganizationSetting.  # noqa: E501
        :type: str
        """

        self._last_updated_by = last_updated_by

    @property
    def created_by(self):
        """Gets the created_by of this OrganizationSetting.  # noqa: E501


        :return: The created_by of this OrganizationSetting.  # noqa: E501
        :rtype: str
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this OrganizationSetting.


        :param created_by: The created_by of this OrganizationSetting.  # noqa: E501
        :type: str
        """

        self._created_by = created_by

    @property
    def created_at(self):
        """Gets the created_at of this OrganizationSetting.  # noqa: E501


        :return: The created_at of this OrganizationSetting.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this OrganizationSetting.


        :param created_at: The created_at of this OrganizationSetting.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this OrganizationSetting.  # noqa: E501


        :return: The updated_at of this OrganizationSetting.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this OrganizationSetting.


        :param updated_at: The updated_at of this OrganizationSetting.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    @property
    def archived_at(self):
        """Gets the archived_at of this OrganizationSetting.  # noqa: E501


        :return: The archived_at of this OrganizationSetting.  # noqa: E501
        :rtype: datetime
        """
        return self._archived_at

    @archived_at.setter
    def archived_at(self, archived_at):
        """Sets the archived_at of this OrganizationSetting.


        :param archived_at: The archived_at of this OrganizationSetting.  # noqa: E501
        :type: datetime
        """

        self._archived_at = archived_at

    @property
    def version(self):
        """Gets the version of this OrganizationSetting.  # noqa: E501


        :return: The version of this OrganizationSetting.  # noqa: E501
        :rtype: float
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this OrganizationSetting.


        :param version: The version of this OrganizationSetting.  # noqa: E501
        :type: float
        """

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(OrganizationSetting, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrganizationSetting):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from typing import List

from curia.api.swagger_client import Dataset
from curia.session import Session
from curia.utils.s3 import s3_listdir, get_metadata


def create_dataset_from_s3_path(
        session: Session,
        dataset_type: str,
        dataset_name: str,
        file_type: str,
        description: str,
        s3_path: str,
        organization_id: str,
        tags: List[str] = None,
        addtl_dataset_kwargs: dict = None):
    if addtl_dataset_kwargs is None:
        addtl_dataset_kwargs = {}
    if tags is None:
        tags = []
    object_locations = s3_listdir(s3_path)
    content_length = 0
    metadata = None
    for sub_object in object_locations:
        metadata = get_metadata(sub_object)
        content_length += metadata["ContentLength"]
    dataset = session.api_instance.create_one_base_dataset_controller_dataset(
        Dataset(
            name=dataset_name,
            description=description,
            type=dataset_type,
            file_type=file_type,
            location=s3_path,
            file_content_type=metadata["ContentType"],
            file_size=str(content_length),
            organization_id=organization_id,
            tags=tags,
            **addtl_dataset_kwargs
        )
    )

    return dataset



import functools
import os
import traceback
from multiprocessing import Pipe
from multiprocessing.context import Process
from typing import Dict
from urllib.parse import urlparse

import moto.s3.responses
from curia.mock.s3 import _create_seed_s3_data
from moto.server import DomainDispatcherApplication, create_backend_app
from werkzeug import run_simple

from curia.utils.s3 import AWSIoContext


class MotoServerFailedException(Exception):
    pass


class ExceptionSafeProcess(Process):
    """
    A process subclass that catches exceptions and stores them in a property.
    """
    def __init__(self, *args, **kwargs):
        Process.__init__(self, *args, **kwargs)
        self._pconn, self._cconn = Pipe()
        self._exception = None

    def run(self):
        try:
            Process.run(self)
            self._cconn.send(None)
        except Exception as e:
            tb = traceback.format_exc()
            self._cconn.send((str(e), tb))

    @property
    def exception(self):
        if self._pconn.poll():
            self._exception = self._pconn.recv()
        return self._exception


def _wrap_subdomain_based_buckets(subdomain_based_buckets):
    """
    Decorator that enables moto3 to correctly identify host.docker.internal as being a path similar to localhost.
    This is necessary because the requests coming from a docker container are not recognized as localhost.
    """
    @functools.wraps(subdomain_based_buckets)
    def wrapper(self, request):
        host = request.headers.get("host", request.headers.get("Host"))
        if not host:
            host = urlparse(request.url).netloc
        if host.startswith("host.docker.internal"):
            return False
        return subdomain_based_buckets(self, request)

    return wrapper


# patch moto with decorator
moto.s3.responses.S3Response.subdomain_based_buckets = \
    _wrap_subdomain_based_buckets(moto.s3.responses.S3Response.subdomain_based_buckets)


def run_moto_server(port=5000):
    """
    Run a moto server in a separate process.
    """
    # WK:
    # Python has a notorious unsolved bug when spinning up a process that uses UrlLib in another thread. In particular,
    # system frameworks using UrlLib make calls to libdispatch to query the system configuration for network proxies.
    # libdispatch is not thread-safe, so this leads to a segfault when used in a separate process.
    # Since this is a test server that does not need any network proxies, we disable it here.
    # See these two python bug threads for more info: https://bugs.python.org/issue30385
    # https://bugs.python.org/issue39853
    os.environ["no_proxy"] = "*"
    main_app = DomainDispatcherApplication(create_backend_app)
    main_app.debug = False
    run_simple(
        hostname="127.0.0.1",
        port=port,
        application=main_app,
        threaded=False,
        use_reloader=False,
        ssl_context=None
    )


def using_moto_server(port=5000, seed_s3_data: Dict[str, dict] = None) -> callable:
    """
    Decorator that starts a moto server in a separate process and kills it after the decorated function has finished.
    """
    def wrap(fn):
        @functools.wraps(fn)
        def wrapper(*args, **kwargs):
            server = ExceptionSafeProcess(
                target=run_moto_server,
                kwargs=dict(port=port),
                daemon=True
            )
            server.start()
            try:
                with AWSIoContext(
                    aws_access_key_id="dummy",
                    aws_secret_access_key="dummy",
                    aws_region="us-east-1",
                    aws_endpoint_url=f"http://localhost:{port}/",
                ):
                    if seed_s3_data:
                        _create_seed_s3_data(seed_s3_data)
                    res = fn(*args, **kwargs)
            finally:
                exception = server.exception
                server.terminate()
                server.join()
                if exception:
                    error, tb = exception
                    raise MotoServerFailedException(
                        f"Moto server failed to start with: {error}.\n Traceback: \n#####{tb}\n#####\n\n"
                    )
            return res

        return wrapper

    return wrap



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.9.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ContainerConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'version': 'str',
        'strategy': 'str',
        'segmentation_strategy': 'str',
        'shap_explanation_count': 'float',
        'attach_shapley_explanation': 'bool',
        'override_targeting_model_bin_count': 'float',
        'override_targeting_method': 'str',
        'save_predictions_to_snowflake': 'bool',
        'snowflake_model_key': 'str',
        'seed': 'float'
    }

    attribute_map = {
        'version': 'version',
        'strategy': 'strategy',
        'segmentation_strategy': 'segmentation_strategy',
        'shap_explanation_count': 'shap_explanation_count',
        'attach_shapley_explanation': 'attach_shapley_explanation',
        'override_targeting_model_bin_count': 'override_targeting_model_bin_count',
        'override_targeting_method': 'override_targeting_method',
        'save_predictions_to_snowflake': 'save_predictions_to_snowflake',
        'snowflake_model_key': 'snowflake_model_key',
        'seed': 'seed'
    }

    def __init__(self, version=None, strategy=None, segmentation_strategy=None, shap_explanation_count=None, attach_shapley_explanation=None, override_targeting_model_bin_count=None, override_targeting_method=None, save_predictions_to_snowflake=None, snowflake_model_key=None, seed=None):  # noqa: E501
        """ContainerConfig - a model defined in Swagger"""  # noqa: E501
        self._version = None
        self._strategy = None
        self._segmentation_strategy = None
        self._shap_explanation_count = None
        self._attach_shapley_explanation = None
        self._override_targeting_model_bin_count = None
        self._override_targeting_method = None
        self._save_predictions_to_snowflake = None
        self._snowflake_model_key = None
        self._seed = None
        self.discriminator = None
        if version is not None:
            self.version = version
        if strategy is not None:
            self.strategy = strategy
        if segmentation_strategy is not None:
            self.segmentation_strategy = segmentation_strategy
        if shap_explanation_count is not None:
            self.shap_explanation_count = shap_explanation_count
        if attach_shapley_explanation is not None:
            self.attach_shapley_explanation = attach_shapley_explanation
        if override_targeting_model_bin_count is not None:
            self.override_targeting_model_bin_count = override_targeting_model_bin_count
        if override_targeting_method is not None:
            self.override_targeting_method = override_targeting_method
        if save_predictions_to_snowflake is not None:
            self.save_predictions_to_snowflake = save_predictions_to_snowflake
        if snowflake_model_key is not None:
            self.snowflake_model_key = snowflake_model_key
        if seed is not None:
            self.seed = seed

    @property
    def version(self):
        """Gets the version of this ContainerConfig.  # noqa: E501


        :return: The version of this ContainerConfig.  # noqa: E501
        :rtype: str
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this ContainerConfig.


        :param version: The version of this ContainerConfig.  # noqa: E501
        :type: str
        """

        self._version = version

    @property
    def strategy(self):
        """Gets the strategy of this ContainerConfig.  # noqa: E501


        :return: The strategy of this ContainerConfig.  # noqa: E501
        :rtype: str
        """
        return self._strategy

    @strategy.setter
    def strategy(self, strategy):
        """Sets the strategy of this ContainerConfig.


        :param strategy: The strategy of this ContainerConfig.  # noqa: E501
        :type: str
        """

        self._strategy = strategy

    @property
    def segmentation_strategy(self):
        """Gets the segmentation_strategy of this ContainerConfig.  # noqa: E501


        :return: The segmentation_strategy of this ContainerConfig.  # noqa: E501
        :rtype: str
        """
        return self._segmentation_strategy

    @segmentation_strategy.setter
    def segmentation_strategy(self, segmentation_strategy):
        """Sets the segmentation_strategy of this ContainerConfig.


        :param segmentation_strategy: The segmentation_strategy of this ContainerConfig.  # noqa: E501
        :type: str
        """

        self._segmentation_strategy = segmentation_strategy

    @property
    def shap_explanation_count(self):
        """Gets the shap_explanation_count of this ContainerConfig.  # noqa: E501


        :return: The shap_explanation_count of this ContainerConfig.  # noqa: E501
        :rtype: float
        """
        return self._shap_explanation_count

    @shap_explanation_count.setter
    def shap_explanation_count(self, shap_explanation_count):
        """Sets the shap_explanation_count of this ContainerConfig.


        :param shap_explanation_count: The shap_explanation_count of this ContainerConfig.  # noqa: E501
        :type: float
        """

        self._shap_explanation_count = shap_explanation_count

    @property
    def attach_shapley_explanation(self):
        """Gets the attach_shapley_explanation of this ContainerConfig.  # noqa: E501


        :return: The attach_shapley_explanation of this ContainerConfig.  # noqa: E501
        :rtype: bool
        """
        return self._attach_shapley_explanation

    @attach_shapley_explanation.setter
    def attach_shapley_explanation(self, attach_shapley_explanation):
        """Sets the attach_shapley_explanation of this ContainerConfig.


        :param attach_shapley_explanation: The attach_shapley_explanation of this ContainerConfig.  # noqa: E501
        :type: bool
        """

        self._attach_shapley_explanation = attach_shapley_explanation

    @property
    def override_targeting_model_bin_count(self):
        """Gets the override_targeting_model_bin_count of this ContainerConfig.  # noqa: E501


        :return: The override_targeting_model_bin_count of this ContainerConfig.  # noqa: E501
        :rtype: float
        """
        return self._override_targeting_model_bin_count

    @override_targeting_model_bin_count.setter
    def override_targeting_model_bin_count(self, override_targeting_model_bin_count):
        """Sets the override_targeting_model_bin_count of this ContainerConfig.


        :param override_targeting_model_bin_count: The override_targeting_model_bin_count of this ContainerConfig.  # noqa: E501
        :type: float
        """

        self._override_targeting_model_bin_count = override_targeting_model_bin_count

    @property
    def override_targeting_method(self):
        """Gets the override_targeting_method of this ContainerConfig.  # noqa: E501


        :return: The override_targeting_method of this ContainerConfig.  # noqa: E501
        :rtype: str
        """
        return self._override_targeting_method

    @override_targeting_method.setter
    def override_targeting_method(self, override_targeting_method):
        """Sets the override_targeting_method of this ContainerConfig.


        :param override_targeting_method: The override_targeting_method of this ContainerConfig.  # noqa: E501
        :type: str
        """

        self._override_targeting_method = override_targeting_method

    @property
    def save_predictions_to_snowflake(self):
        """Gets the save_predictions_to_snowflake of this ContainerConfig.  # noqa: E501


        :return: The save_predictions_to_snowflake of this ContainerConfig.  # noqa: E501
        :rtype: bool
        """
        return self._save_predictions_to_snowflake

    @save_predictions_to_snowflake.setter
    def save_predictions_to_snowflake(self, save_predictions_to_snowflake):
        """Sets the save_predictions_to_snowflake of this ContainerConfig.


        :param save_predictions_to_snowflake: The save_predictions_to_snowflake of this ContainerConfig.  # noqa: E501
        :type: bool
        """

        self._save_predictions_to_snowflake = save_predictions_to_snowflake

    @property
    def snowflake_model_key(self):
        """Gets the snowflake_model_key of this ContainerConfig.  # noqa: E501


        :return: The snowflake_model_key of this ContainerConfig.  # noqa: E501
        :rtype: str
        """
        return self._snowflake_model_key

    @snowflake_model_key.setter
    def snowflake_model_key(self, snowflake_model_key):
        """Sets the snowflake_model_key of this ContainerConfig.


        :param snowflake_model_key: The snowflake_model_key of this ContainerConfig.  # noqa: E501
        :type: str
        """

        self._snowflake_model_key = snowflake_model_key

    @property
    def seed(self):
        """Gets the seed of this ContainerConfig.  # noqa: E501


        :return: The seed of this ContainerConfig.  # noqa: E501
        :rtype: float
        """
        return self._seed

    @seed.setter
    def seed(self, seed):
        """Sets the seed of this ContainerConfig.


        :param seed: The seed of this ContainerConfig.  # noqa: E501
        :type: float
        """

        self._seed = seed

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ContainerConfig, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ContainerConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import dataclasses
from io import BytesIO
from typing import ClassVar, Tuple
from urllib.parse import urlparse
import pyarrow.dataset as ds
import boto3
import pandas as pd


@dataclasses.dataclass
class AWSIoContext:
    """
    Context for AWS IO operations, to allow for easy mocking / testing. If no context is set, the default boto3 client
    will be used.
    """
    aws_access_key_id: str = None
    aws_secret_access_key: str = None
    aws_session_token: str = None
    aws_endpoint_url: str = None
    aws_region: str = None
    _current: ClassVar['AWSIoContext'] = None

    @classmethod
    def get_boto3_client(cls) -> boto3.session.Session.client:
        """
        Retrieve a boto3 client. If no context is set, the default boto3 client will be used.
        :return:
        """
        if cls._current is None:
            return boto3.client('s3')
        return cls._current._get_boto3_client()

    def _get_boto3_client(self) -> boto3.session.Session.client:
        """
        Retrieve a boto3 session
        :return:
        """
        kwargs = {}
        if self.aws_access_key_id:
            kwargs['aws_access_key_id'] = self.aws_access_key_id
        if self.aws_secret_access_key:
            kwargs['aws_secret_access_key'] = self.aws_secret_access_key
        if self.aws_session_token:
            kwargs['aws_session_token'] = self.aws_session_token
        if self.aws_endpoint_url:
            kwargs['endpoint_url'] = self.aws_endpoint_url
        if self.aws_region:
            kwargs['region_name'] = self.aws_region

        return boto3.client(
            's3',
            **kwargs
        )

    def __enter__(self):
        AWSIoContext._current = self
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        AWSIoContext._current = None


def validate_s3_uri(s3_uri: str) -> bool:
    """
    Validates an S3 URI
    :param s3_uri: The S3 URI to validate. S3 URIs must be of the form s3://bucket/key
    :return: True if the URI is valid, False otherwise
    """
    if not s3_uri.startswith('s3://'):
        return False
    if s3_uri.count('/') < 3:
        return False
    if s3_uri.endswith('/'):
        return False
    return True


def validate_s3_dir_uri(s3_uri: str) -> bool:
    """
    Validates an S3 URI for a directory in S3
    :param s3_uri: The S3 URI to validate. S3 dir URIs must be of the form s3://bucket/key/
    :return: True if the URI is valid, False otherwise
    """
    if not s3_uri.startswith('s3://'):
        return False
    if s3_uri.count('/') < 3:
        return False
    if not s3_uri.endswith('/'):
        return False
    return True


def s3_uri_to_bucket_key(s3_uri: str) -> Tuple[str, str]:
    """
    Convert an S3 URI to a bucket name and key
    :param s3_uri: S3 URI to convert
    :return: A tuple of (bucket_name, key)
    """
    parsed_url = urlparse(s3_uri)
    bucket_name = parsed_url.netloc
    directory_name = parsed_url.path[1:]
    return bucket_name, directory_name


def get_metadata(s3_uri: str) -> dict:
    """
    Get the metadata of an S3 object
    :param s3_uri: S3 URI
    :return: Metadata
    """
    assert validate_s3_uri(s3_uri), f'Invalid S3 URI: {s3_uri}'
    s3_client = AWSIoContext.get_boto3_client()
    bucket, key = s3_uri_to_bucket_key(s3_uri)
    metadata = s3_client.head_object(
        Bucket=bucket,
        Key=key
    )
    return metadata


def s3_listdir(s3_uri, max_list_len=1000):
    """
    Replicates os.listdir() functionality for boto3. Returns a list of all objects underneath the
    folder `directory_name` in Bucket `bucket_name`, including those in subfolders.

    Note that S3 does not have a concept of folders, so this function will return all objects that begin with the
    supplied s3_uri. This means that if you have a folder structure like:
    - s3://bucket/folder1/folder2/file1
    - s3://bucket/folder1/folder2/file2
    - s3://bucket/folder1/folder3/file3
    And you call s3_listdir(s3://bucket/folder1/fold), you will get back:
    - s3://bucket/folder1/folder2/file1
    - s3://bucket/folder1/folder2/file2
    - s3://bucket/folder1/folder3/file3
    Even though there is no folder called "fold".

    :param s3_uri: S3 uri to list
    :param max_list_len: Max items to return in initial list

    :return: list of strings giving the names of all objects in the directory `directory_name`
    """
    assert validate_s3_dir_uri(s3_uri), f'Invalid S3 dir URI: {s3_uri}'
    client = AWSIoContext.get_boto3_client()
    bucket_name, directory_name = s3_uri_to_bucket_key(s3_uri)
    is_truncated = True
    continuation_token = None
    keys = []
    while is_truncated:
        if continuation_token is None:
            response = client.list_objects_v2(Bucket=bucket_name, Prefix=directory_name, MaxKeys=max_list_len)
            if response['KeyCount'] == 0:
                return []
        else:
            response = client.list_objects_v2(Bucket=bucket_name,
                                              Prefix=directory_name,
                                              MaxKeys=max_list_len,
                                              ContinuationToken=continuation_token)
        is_truncated = response.get('IsTruncated', False)
        continuation_token = response.get('NextContinuationToken', None)
        keys += [obj.get('Key') for obj in response['Contents']]

    return [f"s3://{bucket_name}/{key}" for key in keys]


def s3_new_bucket(bucket: str) -> None:
    """
    Create a new S3 bucket
    :param bucket: name of bucket to create
    :return:
    """
    s3_client = AWSIoContext.get_boto3_client()
    s3_client.create_bucket(
        Bucket=bucket
    )


def s3_put(s3_uri: str, data: bytes, content_type: str = None, metadata: dict = None) -> None:
    """
    Put data to S3
    :param s3_uri: S3 URI
    :param data: Data to put
    :param content_type: Content type
    :param metadata: Metadata
    :return:
    """
    validate_s3_uri(s3_uri)
    s3_client = AWSIoContext.get_boto3_client()
    bucket, key = s3_uri_to_bucket_key(s3_uri)
    kwargs = {"Bucket": bucket, "Key": key, "Body": data}
    if content_type:
        kwargs['ContentType'] = content_type
    if metadata:
        kwargs['Metadata'] = metadata

    s3_client.put_object(
        **kwargs
    )


def s3_get(s3_uri: str) -> bytes:
    """
    Get data from S3
    :param s3_uri: S3 URI
    :return: Data
    """
    validate_s3_uri(s3_uri)
    s3_client = AWSIoContext.get_boto3_client()
    bucket, key = s3_uri_to_bucket_key(s3_uri)
    obj = s3_client.get_object(
        Bucket=bucket,
        Key=key
    )
    return obj['Body'].read()


def load_parquet_files_from_s3(s3_uri: str) -> pd.DataFrame:
    """
    Loads a group of parquet files from S3 into a single dataframe utilizing Pandas.
    Note that we cannot use the S3 URI directly with pandas because moto doesn't support it.

    :param s3_uri: S3 URI to the parquet files
    :return: Pandas dataframe
    """
    assert validate_s3_dir_uri(s3_uri), f'Invalid S3 dir URI: {s3_uri}'
    files = list(s3_listdir(s3_uri))
    dfs = []
    for file in files:
        dfs.append(pd.read_parquet(BytesIO(s3_get(file))))
    return pd.concat(dfs, ignore_index=True)


def get_parquet_file_metadata(s3_uri: str) -> Tuple[int, int]:
    """
    Get the metadata of a parquet file. Note that this is not moto compatible.
    :param s3_uri:
    :return: A tuple of (number of columns, number of rows)
    """
    assert validate_s3_uri(s3_uri), f'Invalid S3 URI: {s3_uri}'
    dataset = ds.dataset(s3_uri)
    n_columns = len(dataset.schema.names)
    total_n_rows = 0
    for batch in dataset.to_batches(columns=[dataset.schema.names[0]], batch_size=500000):
        total_n_rows += batch.num_rows
    return n_columns, total_n_rows




# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.9.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ModelJobJoinedModelPopulationResponseDto(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'cohorts': 'ModelJobJoinedModelPopulationJoinedCohortDefinitionResponseDto',
        'outcome': 'ModelJobJoinedModelPopulationJoinedOutcomeDefinitionResponseDto',
        'intervention': 'ModelJobJoinedModelPopulationJoinedInterventionDefinitionResponseDto',
        'data_query': 'ModelJobJoinedModelPopulationJoinedDataQueryResponseDto',
        'id': 'str',
        'name': 'str',
        'query_count_status': 'str',
        'query_count_error': 'str',
        'query_list_status': 'str',
        'query_list_error': 'str',
        'query_all_status': 'str',
        'query_all_error': 'str',
        'query_count_started_at': 'datetime',
        'query_count_ended_at': 'datetime',
        'query_list_started_at': 'datetime',
        'query_list_ended_at': 'datetime',
        'query_all_started_at': 'datetime',
        'query_all_ended_at': 'datetime',
        'organization_id': 'str',
        'population_id': 'str',
        'data_query_id': 'str',
        'outcome_distribution_histogram_query_id': 'str',
        'created_at': 'datetime',
        'created_by': 'str',
        'updated_at': 'datetime',
        'archived_at': 'datetime',
        'last_updated_by': 'str',
        'version': 'float'
    }

    attribute_map = {
        'cohorts': 'cohorts',
        'outcome': 'outcome',
        'intervention': 'intervention',
        'data_query': 'dataQuery',
        'id': 'id',
        'name': 'name',
        'query_count_status': 'queryCountStatus',
        'query_count_error': 'queryCountError',
        'query_list_status': 'queryListStatus',
        'query_list_error': 'queryListError',
        'query_all_status': 'queryAllStatus',
        'query_all_error': 'queryAllError',
        'query_count_started_at': 'queryCountStartedAt',
        'query_count_ended_at': 'queryCountEndedAt',
        'query_list_started_at': 'queryListStartedAt',
        'query_list_ended_at': 'queryListEndedAt',
        'query_all_started_at': 'queryAllStartedAt',
        'query_all_ended_at': 'queryAllEndedAt',
        'organization_id': 'organizationId',
        'population_id': 'populationId',
        'data_query_id': 'dataQueryId',
        'outcome_distribution_histogram_query_id': 'outcomeDistributionHistogramQueryId',
        'created_at': 'createdAt',
        'created_by': 'createdBy',
        'updated_at': 'updatedAt',
        'archived_at': 'archivedAt',
        'last_updated_by': 'lastUpdatedBy',
        'version': 'version'
    }

    def __init__(self, cohorts=None, outcome=None, intervention=None, data_query=None, id=None, name=None, query_count_status=None, query_count_error=None, query_list_status=None, query_list_error=None, query_all_status=None, query_all_error=None, query_count_started_at=None, query_count_ended_at=None, query_list_started_at=None, query_list_ended_at=None, query_all_started_at=None, query_all_ended_at=None, organization_id=None, population_id=None, data_query_id=None, outcome_distribution_histogram_query_id=None, created_at=None, created_by=None, updated_at=None, archived_at=None, last_updated_by=None, version=None):  # noqa: E501
        """ModelJobJoinedModelPopulationResponseDto - a model defined in Swagger"""  # noqa: E501
        self._cohorts = None
        self._outcome = None
        self._intervention = None
        self._data_query = None
        self._id = None
        self._name = None
        self._query_count_status = None
        self._query_count_error = None
        self._query_list_status = None
        self._query_list_error = None
        self._query_all_status = None
        self._query_all_error = None
        self._query_count_started_at = None
        self._query_count_ended_at = None
        self._query_list_started_at = None
        self._query_list_ended_at = None
        self._query_all_started_at = None
        self._query_all_ended_at = None
        self._organization_id = None
        self._population_id = None
        self._data_query_id = None
        self._outcome_distribution_histogram_query_id = None
        self._created_at = None
        self._created_by = None
        self._updated_at = None
        self._archived_at = None
        self._last_updated_by = None
        self._version = None
        self.discriminator = None
        if cohorts is not None:
            self.cohorts = cohorts
        if outcome is not None:
            self.outcome = outcome
        if intervention is not None:
            self.intervention = intervention
        if data_query is not None:
            self.data_query = data_query
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if query_count_status is not None:
            self.query_count_status = query_count_status
        if query_count_error is not None:
            self.query_count_error = query_count_error
        if query_list_status is not None:
            self.query_list_status = query_list_status
        if query_list_error is not None:
            self.query_list_error = query_list_error
        if query_all_status is not None:
            self.query_all_status = query_all_status
        if query_all_error is not None:
            self.query_all_error = query_all_error
        if query_count_started_at is not None:
            self.query_count_started_at = query_count_started_at
        if query_count_ended_at is not None:
            self.query_count_ended_at = query_count_ended_at
        if query_list_started_at is not None:
            self.query_list_started_at = query_list_started_at
        if query_list_ended_at is not None:
            self.query_list_ended_at = query_list_ended_at
        if query_all_started_at is not None:
            self.query_all_started_at = query_all_started_at
        if query_all_ended_at is not None:
            self.query_all_ended_at = query_all_ended_at
        if organization_id is not None:
            self.organization_id = organization_id
        if population_id is not None:
            self.population_id = population_id
        if data_query_id is not None:
            self.data_query_id = data_query_id
        if outcome_distribution_histogram_query_id is not None:
            self.outcome_distribution_histogram_query_id = outcome_distribution_histogram_query_id
        if created_at is not None:
            self.created_at = created_at
        if created_by is not None:
            self.created_by = created_by
        if updated_at is not None:
            self.updated_at = updated_at
        if archived_at is not None:
            self.archived_at = archived_at
        if last_updated_by is not None:
            self.last_updated_by = last_updated_by
        if version is not None:
            self.version = version

    @property
    def cohorts(self):
        """Gets the cohorts of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The cohorts of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: ModelJobJoinedModelPopulationJoinedCohortDefinitionResponseDto
        """
        return self._cohorts

    @cohorts.setter
    def cohorts(self, cohorts):
        """Sets the cohorts of this ModelJobJoinedModelPopulationResponseDto.


        :param cohorts: The cohorts of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: ModelJobJoinedModelPopulationJoinedCohortDefinitionResponseDto
        """

        self._cohorts = cohorts

    @property
    def outcome(self):
        """Gets the outcome of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The outcome of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: ModelJobJoinedModelPopulationJoinedOutcomeDefinitionResponseDto
        """
        return self._outcome

    @outcome.setter
    def outcome(self, outcome):
        """Sets the outcome of this ModelJobJoinedModelPopulationResponseDto.


        :param outcome: The outcome of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: ModelJobJoinedModelPopulationJoinedOutcomeDefinitionResponseDto
        """

        self._outcome = outcome

    @property
    def intervention(self):
        """Gets the intervention of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The intervention of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: ModelJobJoinedModelPopulationJoinedInterventionDefinitionResponseDto
        """
        return self._intervention

    @intervention.setter
    def intervention(self, intervention):
        """Sets the intervention of this ModelJobJoinedModelPopulationResponseDto.


        :param intervention: The intervention of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: ModelJobJoinedModelPopulationJoinedInterventionDefinitionResponseDto
        """

        self._intervention = intervention

    @property
    def data_query(self):
        """Gets the data_query of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The data_query of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: ModelJobJoinedModelPopulationJoinedDataQueryResponseDto
        """
        return self._data_query

    @data_query.setter
    def data_query(self, data_query):
        """Sets the data_query of this ModelJobJoinedModelPopulationResponseDto.


        :param data_query: The data_query of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: ModelJobJoinedModelPopulationJoinedDataQueryResponseDto
        """

        self._data_query = data_query

    @property
    def id(self):
        """Gets the id of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The id of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ModelJobJoinedModelPopulationResponseDto.


        :param id: The id of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The name of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ModelJobJoinedModelPopulationResponseDto.


        :param name: The name of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def query_count_status(self):
        """Gets the query_count_status of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The query_count_status of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._query_count_status

    @query_count_status.setter
    def query_count_status(self, query_count_status):
        """Sets the query_count_status of this ModelJobJoinedModelPopulationResponseDto.


        :param query_count_status: The query_count_status of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: str
        """

        self._query_count_status = query_count_status

    @property
    def query_count_error(self):
        """Gets the query_count_error of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The query_count_error of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._query_count_error

    @query_count_error.setter
    def query_count_error(self, query_count_error):
        """Sets the query_count_error of this ModelJobJoinedModelPopulationResponseDto.


        :param query_count_error: The query_count_error of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: str
        """

        self._query_count_error = query_count_error

    @property
    def query_list_status(self):
        """Gets the query_list_status of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The query_list_status of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._query_list_status

    @query_list_status.setter
    def query_list_status(self, query_list_status):
        """Sets the query_list_status of this ModelJobJoinedModelPopulationResponseDto.


        :param query_list_status: The query_list_status of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: str
        """

        self._query_list_status = query_list_status

    @property
    def query_list_error(self):
        """Gets the query_list_error of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The query_list_error of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._query_list_error

    @query_list_error.setter
    def query_list_error(self, query_list_error):
        """Sets the query_list_error of this ModelJobJoinedModelPopulationResponseDto.


        :param query_list_error: The query_list_error of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: str
        """

        self._query_list_error = query_list_error

    @property
    def query_all_status(self):
        """Gets the query_all_status of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The query_all_status of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._query_all_status

    @query_all_status.setter
    def query_all_status(self, query_all_status):
        """Sets the query_all_status of this ModelJobJoinedModelPopulationResponseDto.


        :param query_all_status: The query_all_status of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: str
        """

        self._query_all_status = query_all_status

    @property
    def query_all_error(self):
        """Gets the query_all_error of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The query_all_error of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._query_all_error

    @query_all_error.setter
    def query_all_error(self, query_all_error):
        """Sets the query_all_error of this ModelJobJoinedModelPopulationResponseDto.


        :param query_all_error: The query_all_error of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: str
        """

        self._query_all_error = query_all_error

    @property
    def query_count_started_at(self):
        """Gets the query_count_started_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The query_count_started_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._query_count_started_at

    @query_count_started_at.setter
    def query_count_started_at(self, query_count_started_at):
        """Sets the query_count_started_at of this ModelJobJoinedModelPopulationResponseDto.


        :param query_count_started_at: The query_count_started_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: datetime
        """

        self._query_count_started_at = query_count_started_at

    @property
    def query_count_ended_at(self):
        """Gets the query_count_ended_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The query_count_ended_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._query_count_ended_at

    @query_count_ended_at.setter
    def query_count_ended_at(self, query_count_ended_at):
        """Sets the query_count_ended_at of this ModelJobJoinedModelPopulationResponseDto.


        :param query_count_ended_at: The query_count_ended_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: datetime
        """

        self._query_count_ended_at = query_count_ended_at

    @property
    def query_list_started_at(self):
        """Gets the query_list_started_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The query_list_started_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._query_list_started_at

    @query_list_started_at.setter
    def query_list_started_at(self, query_list_started_at):
        """Sets the query_list_started_at of this ModelJobJoinedModelPopulationResponseDto.


        :param query_list_started_at: The query_list_started_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: datetime
        """

        self._query_list_started_at = query_list_started_at

    @property
    def query_list_ended_at(self):
        """Gets the query_list_ended_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The query_list_ended_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._query_list_ended_at

    @query_list_ended_at.setter
    def query_list_ended_at(self, query_list_ended_at):
        """Sets the query_list_ended_at of this ModelJobJoinedModelPopulationResponseDto.


        :param query_list_ended_at: The query_list_ended_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: datetime
        """

        self._query_list_ended_at = query_list_ended_at

    @property
    def query_all_started_at(self):
        """Gets the query_all_started_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The query_all_started_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._query_all_started_at

    @query_all_started_at.setter
    def query_all_started_at(self, query_all_started_at):
        """Sets the query_all_started_at of this ModelJobJoinedModelPopulationResponseDto.


        :param query_all_started_at: The query_all_started_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: datetime
        """

        self._query_all_started_at = query_all_started_at

    @property
    def query_all_ended_at(self):
        """Gets the query_all_ended_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The query_all_ended_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._query_all_ended_at

    @query_all_ended_at.setter
    def query_all_ended_at(self, query_all_ended_at):
        """Sets the query_all_ended_at of this ModelJobJoinedModelPopulationResponseDto.


        :param query_all_ended_at: The query_all_ended_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: datetime
        """

        self._query_all_ended_at = query_all_ended_at

    @property
    def organization_id(self):
        """Gets the organization_id of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The organization_id of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this ModelJobJoinedModelPopulationResponseDto.


        :param organization_id: The organization_id of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: str
        """

        self._organization_id = organization_id

    @property
    def population_id(self):
        """Gets the population_id of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The population_id of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._population_id

    @population_id.setter
    def population_id(self, population_id):
        """Sets the population_id of this ModelJobJoinedModelPopulationResponseDto.


        :param population_id: The population_id of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: str
        """

        self._population_id = population_id

    @property
    def data_query_id(self):
        """Gets the data_query_id of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The data_query_id of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._data_query_id

    @data_query_id.setter
    def data_query_id(self, data_query_id):
        """Sets the data_query_id of this ModelJobJoinedModelPopulationResponseDto.


        :param data_query_id: The data_query_id of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: str
        """

        self._data_query_id = data_query_id

    @property
    def outcome_distribution_histogram_query_id(self):
        """Gets the outcome_distribution_histogram_query_id of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The outcome_distribution_histogram_query_id of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._outcome_distribution_histogram_query_id

    @outcome_distribution_histogram_query_id.setter
    def outcome_distribution_histogram_query_id(self, outcome_distribution_histogram_query_id):
        """Sets the outcome_distribution_histogram_query_id of this ModelJobJoinedModelPopulationResponseDto.


        :param outcome_distribution_histogram_query_id: The outcome_distribution_histogram_query_id of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: str
        """

        self._outcome_distribution_histogram_query_id = outcome_distribution_histogram_query_id

    @property
    def created_at(self):
        """Gets the created_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The created_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this ModelJobJoinedModelPopulationResponseDto.


        :param created_at: The created_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def created_by(self):
        """Gets the created_by of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The created_by of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this ModelJobJoinedModelPopulationResponseDto.


        :param created_by: The created_by of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: str
        """

        self._created_by = created_by

    @property
    def updated_at(self):
        """Gets the updated_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The updated_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this ModelJobJoinedModelPopulationResponseDto.


        :param updated_at: The updated_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    @property
    def archived_at(self):
        """Gets the archived_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The archived_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._archived_at

    @archived_at.setter
    def archived_at(self, archived_at):
        """Sets the archived_at of this ModelJobJoinedModelPopulationResponseDto.


        :param archived_at: The archived_at of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: datetime
        """

        self._archived_at = archived_at

    @property
    def last_updated_by(self):
        """Gets the last_updated_by of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The last_updated_by of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, last_updated_by):
        """Sets the last_updated_by of this ModelJobJoinedModelPopulationResponseDto.


        :param last_updated_by: The last_updated_by of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: str
        """

        self._last_updated_by = last_updated_by

    @property
    def version(self):
        """Gets the version of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501


        :return: The version of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :rtype: float
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this ModelJobJoinedModelPopulationResponseDto.


        :param version: The version of this ModelJobJoinedModelPopulationResponseDto.  # noqa: E501
        :type: float
        """

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ModelJobJoinedModelPopulationResponseDto, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ModelJobJoinedModelPopulationResponseDto):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import functools
from typing import Callable, Dict, Any

import mlflow


class MLFlowLogger:
    def __init__(self, run_name=None, tags=None, autolog=False, is_nested=False):
        self.run_name = run_name
        self.tags = tags
        self.autolog = autolog
        self.is_nested = is_nested

    @staticmethod
    def log_dict(dict_to_log):
        for key, value in dict_to_log.items():
            mlflow.log_metric(key, value)

    def __call__(self, func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            if self.autolog:
                mlflow.autolog(log_datasets=False, log_models=True)
            with mlflow.start_run(
                nested=self.is_nested, run_name=self.run_name, tags=self.tags
            ):
                result = func(*args, **kwargs)

                # If the function returns a dictionary, log it.
                if isinstance(result, dict):
                    self.log_dict(result)

            return result

        return wrapper


def using_mlflow_logging(
    run_name: str = None,
    tags: Dict[str, Any] = None,
    autolog: bool = False,
    is_nested: bool = False,
) -> Callable:
    """
    A convenience function for using the MLFlowLogger class as a decorator.

    Args:
        run_name (str, optional): The name of the run. Defaults to None.
        tags (Dict[str, Any], optional): A dictionary of tags to add to the run. Defaults to None.
        autolog (bool, optional): Whether to use MLFlow's autologging feature. Defaults to False.
        is_nested (bool, optional): Whether the run is nested within another run. Defaults to False.

    Returns:
        Callable: A decorator for logging MLFlow runs.
    """

    logger = MLFlowLogger(run_name, tags, autolog, is_nested)

    def decorator(func: Callable) -> Callable:
        return logger(func)

    return decorator



from typing import Dict, Any, Callable

import mlflow


def make_predictions(model: Any, data: Any, prediction_function: Callable) -> Any:
    """
    Makes predictions using the provided model and data.

    Parameters:
        model: The model to be used.
        data: The data on which to make predictions.
        prediction_function: The function to use for making predictions. This function should take a model and a dataset
        as inputs and return predictions.

    Returns:
        The predictions made by the model on the data.

    Example:
        The `prediction_function` could be a function like the following:
        def predict(model, data):
            return model.predict(data)

    """
    return prediction_function(model, data)


def evaluate_predictions(
    y_true: Any, y_pred: Any, metric_func: Callable
) -> Dict[str, float]:
    """
    Evaluates predictions using the provided metric function.

    Parameters:
        y_true: The true target values.
        y_pred: The predicted target values.
        metric_func: The function used to compute the metrics. This function should take true and predicted values as
        inputs and return a dictionary mapping metric names to computed metric values.

    Returns:
        A dictionary mapping metric names to computed metric values.
        Example: {'log_loss': 0.123, 'roc_auc': 0.456}

    Example:
        The `metric_func` could be a function like the following:

        def compute_metrics(y_true, y_pred):
            precision = metrics.precision_score(y_true, y_pred)
            recall = metrics.recall_score(y_true, y_pred)

            return {"precision": precision, "recall": recall}
    """
    return metric_func(y_true, y_pred)


def evaluate_model_on_datasets(
    model: Any,
    datasets: Dict[str, Any],
    prediction_function: Callable,
    metric_func: Callable,
    label_col: str = "label",
) -> Dict[str, float]:
    """
    Evaluates a model's performance across multiple datasets using a specified metric function.

    Parameters:
        model: The model to be evaluated.
        datasets: A dictionary mapping dataset names to datasets.
        prediction_function: The function to use for making predictions. This function should take a model and a dataset
        as inputs and return predictions.
        metric_func: The function used to compute the metrics. This function should take true and predicted values as
        inputs and return a dictionary mapping metric names to computed metric values.
        label_col: The name of the column containing the true target values.

    Returns:
        A dictionary mapping "{dataset_name}_{metric_name}" to computed metric values.
    """

    results = {}

    for name, data in datasets.items():
        # Make predictions
        y_true = data[label_col]
        y_pred = make_predictions(
            model, data.drop(columns=[label_col]), prediction_function
        )

        # Compute metrics
        metrics = evaluate_predictions(y_true, y_pred, metric_func)

        # Store results in dictionary
        for metric_name, metric_value in metrics.items():
            results[f"{name}_{metric_name}"] = metric_value

    return results


def get_mlflow_run(id_value, id_type="runName"):
    """
    Retrieve the MLFlow run with the specified ID value and type.

    :param id_value: The value of the ID to search for.
    :param id_type: The type of the ID to search for.
    to 'runName'.
    :return: The MLFlow run with the specified ID value and type.
    """
    assert id_type in ["runName", "parentRunId"], "Invalid ID type"

    # Use mlflow.search_runs to find the run by its ID
    runs = mlflow.search_runs(filter_string=f"tags.mlflow.{id_type}='{id_value}'")
    assert len(runs) == 1, "The number of runs with the specified ID is not exactly 1"

    # Return the first (and only) run
    return runs.iloc[0]
